# Generated by Django 5.1.3 on 2024-12-11 18:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_goals', '0005_goal_goals_waiting_for_precond_idx'),
    ]

    operations = [
        migrations.AddField(
            model_name='goal',
            name='waiting_for_failed_count',
            field=models.IntegerField(default=0, help_text='Number of precondition goals that failed.'),
        ),
        migrations.AddIndex(
            model_name='goal',
            index=models.Index(condition=models.Q(('state', 'waiting_for_preconditions')), fields=['waiting_for_failed_count'], name='goals_waiting_for_failed_idx'),
        ),
        migrations.AddIndex(
            model_name='goal',
            index=models.Index(condition=models.Q(('state', 'not_going_to_happen_soon')), fields=['waiting_for_failed_count'], name='goals_unblocking_idx'),
        ),
    ]
