# Generated by Django 5.1.3 on 2024-12-08 00:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_goals', '0003_remove_goal_goals_waiting_for_worker_idx_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='goal',
            name='waiting_for_count',
            field=models.IntegerField(
                default=0,
                help_text='Number of precondition goals that must finish before this goal can be pursued.',
            ),
        ),
    ]
