# Generated by Django 5.0.7 on 2024-09-10 16:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_goals', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='goalprogress',
            name='message',
            field=models.CharField(blank=True, max_length=200),
        ),
    ]
