# Generated by Django 5.1.4 on 2025-01-11 15:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_goals', '0008_remove_goal_goals_waiting_for_precond_idx_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='goal',
            name='waiting_for_not_achieved_count',
            field=models.IntegerField(default=0, help_text='Number of precondition goals that are not achieved yet.'),
        ),
    ]
